@@ -7,6 +7,7 @@ use log::*;
 use smol::channel::Receiver as SmolReceiver;
 use std::{
     collections::BTreeMap,
+    fmt,
     io::{self, Write},
     net::{IpAddr, SocketAddr},
     path::PathBuf,
@@ -17,6 +18,38 @@ use tokio::sync::{mpsc, Mutex};
 use wezterm_ssh::{Config as WezConfig, Session as WezSession, SessionEvent as WezSessionEvent};
 
 mod handler;
+mod process;
+
+/// Represents the backend to use for ssh operations
+#[derive(Copy, Clone, Debug, PartialEq, Eq)]
+#[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
+#[cfg_attr(feature = "serde", serde(rename_all = "lowercase"))]
+pub enum SshBackend {
+    /// Use libssh as backend
+    LibSsh,
+
+    /// Use ssh2 as backend
+    Ssh2,
+}
+
+impl Default for SshBackend {
+    /// Defaults to ssh2
+    ///
+    /// NOTE: There are currently bugs in libssh that cause our implementation to hang related to
+    ///       process stdout/stderr and maybe other logic.
+    fn default() -> Self {
+        Self::Ssh2
+    }
+}
+
+impl fmt::Display for SshBackend {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        match self {
+            Self::LibSsh => write!(f, "libssh"),
+            Self::Ssh2 => write!(f, "ssh2"),
+        }
+    }
+}
 
 /// Represents a singular authentication prompt for a new ssh session
 #[derive(Debug)]
@@ -50,6 +83,9 @@ pub struct Ssh2AuthEvent {
 #[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
 #[cfg_attr(feature = "serde", serde(default))]
 pub struct Ssh2SessionOpts {
+    /// Represents the backend to use for ssh operations
+    pub backend: SshBackend,
+
     /// List of files from which the user's DSA, ECDSA, Ed25519, or RSA authentication identity
     /// is read, defaulting to
     ///
@@ -80,6 +116,9 @@ pub struct Ssh2SessionOpts {
     /// - `~/.ssh/known_hosts2`
     pub user_known_hosts_files: Vec<PathBuf>,
 
+    /// If true, will output tracing information from the underlying ssh implementation
+    pub verbose: bool,
+
     /// Additional options to provide as defined by `ssh_config(5)`
     pub other: BTreeMap<String, String>,
 }
@@ -249,6 +288,12 @@ impl Ssh2Session {
             );
         }
 
+        // Set verbosity optin for ssh lib
+        config.insert("wezterm_ssh_verbose".to_string(), opts.verbose.to_string());
+
+        // Set the backend to use going forward
+        config.insert("wezterm_ssh_backend".to_string(), opts.backend.to_string());
+
         // Add in any of the other options provided
         config.extend(opts.other);
 
@@ -357,9 +402,24 @@ impl Ssh2Session {
             ));
         }
 
+        let timeout = opts.timeout;
+
         // Determine distinct candidate ip addresses for connecting
+        //
+        // NOTE: This breaks when the host is an alias defined within an ssh config; however,
+        //       we need to be able to resolve the IP address(es) for use in TCP connect. The
+        //       end solution would be to have wezterm-ssh provide some means to determine the
+        //       IP address of the end machine it is connected to, but that probably isn't
+        //       possible with ssh. So, for now, connecting to a distant server from an
+        //       established ssh connection requires that we can resolve the specified host
         let mut candidate_ips = tokio::net::lookup_host(format!("{}:{}", self.host, self.port))
-            .await?
+            .await
+            .map_err(|x| {
+                io::Error::new(
+                    x.kind(),
+                    format!("{} needs to be resolvable outside of ssh: {}", self.host, x),
+                )
+            })?
             .into_iter()
             .map(|addr| addr.ip())
             .collect::<Vec<IpAddr>>();
@@ -372,6 +432,41 @@ impl Ssh2Session {
             ));
         }
 
+        let info = self.into_distant_session_info(opts).await?;
+        let key = info.key;
+        let codec = XChaCha20Poly1305Codec::from(key);
+
+        // Try each IP address with the same port to see if one works
+        let mut err = None;
+        for ip in candidate_ips {
+            let addr = SocketAddr::new(ip, info.port);
+            debug!("Attempting to connect to distant server @ {}", addr);
+            match Session::tcp_connect_timeout(addr, codec.clone(), timeout).await {
+                Ok(session) => return Ok(session),
+                Err(x) => err = Some(x),
+            }
+        }
+
+        // If all failed, return the last error we got
+        Err(err.expect("Err set above"))
+    }
+
+    /// Consume [`Ssh2Session`] and produce a distant [`SessionInfo`] representing a remote
+    /// distant server that is spawned using the ssh session
+    pub async fn into_distant_session_info(
+        self,
+        opts: IntoDistantSessionOpts,
+    ) -> io::Result<SessionInfo> {
+        // Exit early if not authenticated as this is a requirement
+        if !self.authenticated {
+            return Err(io::Error::new(
+                io::ErrorKind::PermissionDenied,
+                "Not authenticated",
+            ));
+        }
+
+        let host = self.host().to_string();
+
         // Turn our ssh connection into a client session so we can use it to spawn our server
         let mut session = self.into_ssh_client_session().await?;
 
@@ -409,7 +504,7 @@ impl Ssh2Session {
         // ssh session is closed
         debug!("Executing {} {}", bin, args.join(" "));
         let mut proc = session
-            .spawn("<ssh-launch>", bin, args, true)
+            .spawn("<ssh-launch>", bin, args, true, None)
             .await
             .map_err(|x| io::Error::new(io::ErrorKind::Other, x))?;
         let mut stdout = proc.stdout.take().unwrap();
@@ -422,37 +517,24 @@ impl Ssh2Session {
         // Close out ssh session
         session.abort();
         let _ = session.wait().await;
-        let mut output = String::new();
+        let mut output = Vec::new();
 
         // If successful, grab the session information and establish a connection
         // with the distant server
         if success {
             while let Ok(data) = stdout.read().await {
-                output.push_str(&data);
+                output.extend(&data);
             }
 
+            // Iterate over output as individual lines, looking for session info
             let maybe_info = output
-                .lines()
+                .split(|&b| b == b'\n')
+                .map(String::from_utf8_lossy)
                 .find_map(|line| line.parse::<SessionInfo>().ok());
             match maybe_info {
-                Some(info) => {
-                    let key = info.key;
-                    let codec = XChaCha20Poly1305Codec::from(key);
-
-                    // Try each IP address with the same port to see if one works
-                    let mut err = None;
-                    for ip in candidate_ips {
-                        let addr = SocketAddr::new(ip, info.port);
-                        debug!("Attempting to connect to distant server @ {}", addr);
-                        match Session::tcp_connect_timeout(addr, codec.clone(), opts.timeout).await
-                        {
-                            Ok(session) => return Ok(session),
-                            Err(x) => err = Some(x),
-                        }
-                    }
-
-                    // If all failed, return the last error we got
-                    Err(err.expect("Err set above"))
+                Some(mut info) => {
+                    info.host = host;
+                    Ok(info)
                 }
                 None => Err(io::Error::new(
                     io::ErrorKind::InvalidData,
@@ -461,7 +543,7 @@ impl Ssh2Session {
             }
         } else {
             while let Ok(data) = stderr.read().await {
-                output.push_str(&data);
+                output.extend(&data);
             }
 
             Err(io::Error::new(
@@ -470,7 +552,10 @@ impl Ssh2Session {
                     "Spawning distant failed [{}]: {}",
                     code.map(|x| x.to_string())
                         .unwrap_or_else(|| String::from("???")),
-                    output
+                    match String::from_utf8(output) {
+                        Ok(output) => output,
+                        Err(x) => x.to_string(),
+                    }
                 ),
             ))
         }
@@ -488,19 +573,8 @@ impl Ssh2Session {
         }
 
         let (t1, t2) = Transport::pair(1);
-        let addr = tokio::net::lookup_host(format!("{}:{}", self.host, self.port))
-            .await?
-            .next()
-            .ok_or_else(|| {
-                io::Error::new(
-                    io::ErrorKind::AddrNotAvailable,
-                    format!("Failed to resolve host: {}", self.host),
-                )
-            })?;
-        let tag = t1.to_connection_tag();
-
-        let session =
-            Session::initialize_with_details(t1, Some(SessionDetails::Tcp { addr, tag }))?;
+        let tag = format!("ssh {}:{}", self.host, self.port);
+        let session = Session::initialize_with_details(t1, Some(SessionDetails::Custom { tag }))?;
 
         // Spawn tasks that forward requests to the ssh session
         // and send back responses from the ssh session
