@@ -1123,7 +1123,7 @@ async fn exists_should_send_true_if_path_exists(#[future] session: Session) {
 
     let res = session.send(req).await.unwrap();
     assert_eq!(res.payload.len(), 1, "Wrong payload size");
-    assert_eq!(res.payload[0], ResponseData::Exists(true));
+    assert_eq!(res.payload[0], ResponseData::Exists { value: true });
 }
 
 #[rstest]
@@ -1142,7 +1142,7 @@ async fn exists_should_send_false_if_path_does_not_exist(#[future] session: Sess
 
     let res = session.send(req).await.unwrap();
     assert_eq!(res.payload.len(), 1, "Wrong payload size");
-    assert_eq!(res.payload[0], ResponseData::Exists(false));
+    assert_eq!(res.payload[0], ResponseData::Exists { value: false });
 }
 
 #[rstest]
@@ -1348,14 +1348,15 @@ async fn metadata_should_resolve_file_type_of_symlink_if_flag_specified(
 
 #[rstest]
 #[tokio::test]
-async fn proc_run_should_send_error_over_stderr_on_failure(#[future] session: Session) {
+async fn proc_spawn_should_send_error_over_stderr_on_failure(#[future] session: Session) {
     let mut session = session.await;
     let req = Request::new(
         "test-tenant",
-        vec![RequestData::ProcRun {
+        vec![RequestData::ProcSpawn {
             cmd: DOES_NOT_EXIST_BIN.to_str().unwrap().to_string(),
             args: Vec::new(),
-            detached: false,
+            persist: false,
+            pty: None,
         }],
     );
 
@@ -1367,7 +1368,7 @@ async fn proc_run_should_send_error_over_stderr_on_failure(#[future] session: Se
     let res = mailbox.next().await.unwrap();
     assert_eq!(res.payload.len(), 1, "Wrong payload size");
     let proc_id = match &res.payload[0] {
-        ResponseData::ProcStart { id } => *id,
+        ResponseData::ProcSpawned { id } => *id,
         x => panic!("Unexpected response: {:?}", x),
     };
 
@@ -1394,21 +1395,22 @@ async fn proc_run_should_send_error_over_stderr_on_failure(#[future] session: Se
 
 #[rstest]
 #[tokio::test]
-async fn proc_run_should_send_back_proc_start_on_success(#[future] session: Session) {
+async fn proc_spawn_should_send_back_proc_start_on_success(#[future] session: Session) {
     let mut session = session.await;
     let req = Request::new(
         "test-tenant",
-        vec![RequestData::ProcRun {
+        vec![RequestData::ProcSpawn {
             cmd: SCRIPT_RUNNER.to_string(),
             args: vec![ECHO_ARGS_TO_STDOUT_SH.to_str().unwrap().to_string()],
-            detached: false,
+            persist: false,
+            pty: None,
         }],
     );
 
     let res = session.send(req).await.unwrap();
     assert_eq!(res.payload.len(), 1, "Wrong payload size");
     assert!(
-        matches!(&res.payload[0], ResponseData::ProcStart { .. }),
+        matches!(&res.payload[0], ResponseData::ProcSpawned { .. }),
         "Unexpected response: {:?}",
         res.payload[0]
     );
@@ -1419,18 +1421,21 @@ async fn proc_run_should_send_back_proc_start_on_success(#[future] session: Sess
 #[rstest]
 #[tokio::test]
 #[cfg_attr(windows, ignore)]
-async fn proc_run_should_send_back_stdout_periodically_when_available(#[future] session: Session) {
+async fn proc_spawn_should_send_back_stdout_periodically_when_available(
+    #[future] session: Session,
+) {
     let mut session = session.await;
     // Run a program that echoes to stdout
     let req = Request::new(
         "test-tenant",
-        vec![RequestData::ProcRun {
+        vec![RequestData::ProcSpawn {
             cmd: SCRIPT_RUNNER.to_string(),
             args: vec![
                 ECHO_ARGS_TO_STDOUT_SH.to_str().unwrap().to_string(),
                 String::from("'some stdout'"),
             ],
-            detached: false,
+            persist: false,
+            pty: None,
         }],
     );
 
@@ -1439,7 +1444,7 @@ async fn proc_run_should_send_back_stdout_periodically_when_available(#[future]
     let res = mailbox.next().await.unwrap();
     assert_eq!(res.payload.len(), 1, "Wrong payload size");
     assert!(
-        matches!(&res.payload[0], ResponseData::ProcStart { .. }),
+        matches!(&res.payload[0], ResponseData::ProcSpawned { .. }),
         "Unexpected response: {:?}",
         res.payload[0]
     );
@@ -1461,7 +1466,7 @@ async fn proc_run_should_send_back_stdout_periodically_when_available(#[future]
         assert_eq!(res.payload.len(), 1, "Wrong payload size");
         match &res.payload[0] {
             ResponseData::ProcStdout { data, .. } => {
-                assert_eq!(data, "some stdout", "Got wrong stdout");
+                assert_eq!(data, b"some stdout", "Got wrong stdout");
                 got_stdout = true;
             }
             ResponseData::ProcDone { success, .. } => {
@@ -1483,18 +1488,21 @@ async fn proc_run_should_send_back_stdout_periodically_when_available(#[future]
 #[rstest]
 #[tokio::test]
 #[cfg_attr(windows, ignore)]
-async fn proc_run_should_send_back_stderr_periodically_when_available(#[future] session: Session) {
+async fn proc_spawn_should_send_back_stderr_periodically_when_available(
+    #[future] session: Session,
+) {
     let mut session = session.await;
     // Run a program that echoes to stderr
     let req = Request::new(
         "test-tenant",
-        vec![RequestData::ProcRun {
+        vec![RequestData::ProcSpawn {
             cmd: SCRIPT_RUNNER.to_string(),
             args: vec![
                 ECHO_ARGS_TO_STDERR_SH.to_str().unwrap().to_string(),
                 String::from("'some stderr'"),
             ],
-            detached: false,
+            persist: false,
+            pty: None,
         }],
     );
 
@@ -1503,7 +1511,7 @@ async fn proc_run_should_send_back_stderr_periodically_when_available(#[future]
     let res = mailbox.next().await.unwrap();
     assert_eq!(res.payload.len(), 1, "Wrong payload size");
     assert!(
-        matches!(&res.payload[0], ResponseData::ProcStart { .. }),
+        matches!(&res.payload[0], ResponseData::ProcSpawned { .. }),
         "Unexpected response: {:?}",
         res.payload[0]
     );
@@ -1525,7 +1533,7 @@ async fn proc_run_should_send_back_stderr_periodically_when_available(#[future]
         assert_eq!(res.payload.len(), 1, "Wrong payload size");
         match &res.payload[0] {
             ResponseData::ProcStderr { data, .. } => {
-                assert_eq!(data, "some stderr", "Got wrong stderr");
+                assert_eq!(data, b"some stderr", "Got wrong stderr");
                 got_stderr = true;
             }
             ResponseData::ProcDone { success, .. } => {
@@ -1547,15 +1555,16 @@ async fn proc_run_should_send_back_stderr_periodically_when_available(#[future]
 #[rstest]
 #[tokio::test]
 #[cfg_attr(windows, ignore)]
-async fn proc_run_should_clear_process_from_state_when_done(#[future] session: Session) {
+async fn proc_spawn_should_clear_process_from_state_when_done(#[future] session: Session) {
     let mut session = session.await;
     // Run a program that ends after a little bit
     let req = Request::new(
         "test-tenant",
-        vec![RequestData::ProcRun {
+        vec![RequestData::ProcSpawn {
             cmd: SCRIPT_RUNNER.to_string(),
             args: vec![SLEEP_SH.to_str().unwrap().to_string(), String::from("0.1")],
-            detached: false,
+            persist: false,
+            pty: None,
         }],
     );
     let mut mailbox = session.mail(req).await.unwrap();
@@ -1563,7 +1572,7 @@ async fn proc_run_should_clear_process_from_state_when_done(#[future] session: S
     let res = mailbox.next().await.unwrap();
     assert_eq!(res.payload.len(), 1, "Wrong payload size");
     let id = match &res.payload[0] {
-        ResponseData::ProcStart { id } => *id,
+        ResponseData::ProcSpawned { id } => *id,
         x => panic!("Unexpected response: {:?}", x),
     };
 
@@ -1595,15 +1604,16 @@ async fn proc_run_should_clear_process_from_state_when_done(#[future] session: S
 
 #[rstest]
 #[tokio::test]
-async fn proc_run_should_clear_process_from_state_when_killed(#[future] session: Session) {
+async fn proc_spawn_should_clear_process_from_state_when_killed(#[future] session: Session) {
     let mut session = session.await;
     // Run a program that ends slowly
     let req = Request::new(
         "test-tenant",
-        vec![RequestData::ProcRun {
+        vec![RequestData::ProcSpawn {
             cmd: SCRIPT_RUNNER.to_string(),
             args: vec![SLEEP_SH.to_str().unwrap().to_string(), String::from("1")],
-            detached: false,
+            persist: false,
+            pty: None,
         }],
     );
 
@@ -1612,7 +1622,7 @@ async fn proc_run_should_clear_process_from_state_when_killed(#[future] session:
     let res = mailbox.next().await.unwrap();
     assert_eq!(res.payload.len(), 1, "Wrong payload size");
     let id = match &res.payload[0] {
-        ResponseData::ProcStart { id } => *id,
+        ResponseData::ProcSpawned { id } => *id,
         x => panic!("Unexpected response: {:?}", x),
     };
 
@@ -1674,10 +1684,11 @@ async fn proc_kill_should_send_ok_and_done_responses_on_success(#[future] sessio
     // First, run a program that sits around (sleep for 1 second)
     let req = Request::new(
         "test-tenant",
-        vec![RequestData::ProcRun {
+        vec![RequestData::ProcSpawn {
             cmd: SCRIPT_RUNNER.to_string(),
             args: vec![SLEEP_SH.to_str().unwrap().to_string(), String::from("1")],
-            detached: false,
+            persist: false,
+            pty: None,
         }],
     );
 
@@ -1688,7 +1699,7 @@ async fn proc_kill_should_send_ok_and_done_responses_on_success(#[future] sessio
 
     // Second, grab the id of the started process
     let id = match &res.payload[0] {
-        ResponseData::ProcStart { id } => *id,
+        ResponseData::ProcSpawned { id } => *id,
         x => panic!("Unexpected response: {:?}", x),
     };
 
@@ -1721,7 +1732,7 @@ async fn proc_stdin_should_send_error_on_failure(#[future] session: Session) {
         "test-tenant",
         vec![RequestData::ProcStdin {
             id: 0xDEADBEEF,
-            data: String::from("some input"),
+            data: b"some input".to_vec(),
         }],
     );
 
@@ -1749,10 +1760,11 @@ async fn proc_stdin_should_send_ok_on_success_and_properly_send_stdin_to_process
     // First, run a program that listens for stdin
     let req = Request::new(
         "test-tenant",
-        vec![RequestData::ProcRun {
+        vec![RequestData::ProcSpawn {
             cmd: SCRIPT_RUNNER.to_string(),
             args: vec![ECHO_STDIN_TO_STDOUT_SH.to_str().unwrap().to_string()],
-            detached: false,
+            persist: false,
+            pty: None,
         }],
     );
     let mut mailbox = session.mail(req).await.unwrap();
@@ -1762,7 +1774,7 @@ async fn proc_stdin_should_send_ok_on_success_and_properly_send_stdin_to_process
 
     // Second, grab the id of the started process
     let id = match &res.payload[0] {
-        ResponseData::ProcStart { id } => *id,
+        ResponseData::ProcSpawned { id } => *id,
         x => panic!("Unexpected response: {:?}", x),
     };
 
@@ -1773,7 +1785,7 @@ async fn proc_stdin_should_send_ok_on_success_and_properly_send_stdin_to_process
         "test-tenant",
         vec![RequestData::ProcStdin {
             id,
-            data: String::from("hello world\n"),
+            data: b"hello world\n".to_vec(),
         }],
     );
     let res = session.send(req).await.unwrap();
@@ -1786,7 +1798,7 @@ async fn proc_stdin_should_send_ok_on_success_and_properly_send_stdin_to_process
     let res = mailbox.next().await.unwrap();
     match &res.payload[0] {
         ResponseData::ProcStdout { data, .. } => {
-            assert_eq!(data, "hello world\n", "Mirrored data didn't match");
+            assert_eq!(data, b"hello world\n", "Mirrored data didn't match");
         }
         x => panic!("Unexpected response: {:?}", x),
     }
@@ -1798,10 +1810,11 @@ async fn proc_list_should_send_proc_entry_list(#[future] session: Session) {
     let mut session = session.await;
     let req = Request::new(
         "test-tenant",
-        vec![RequestData::ProcRun {
+        vec![RequestData::ProcSpawn {
             cmd: SCRIPT_RUNNER.to_string(),
             args: vec![SLEEP_SH.to_str().unwrap().to_string(), String::from("10")],
-            detached: false,
+            persist: false,
+            pty: None,
         }],
     );
 
@@ -1810,7 +1823,7 @@ async fn proc_list_should_send_proc_entry_list(#[future] session: Session) {
 
     // Grab the id of the started process
     let id = match &res.payload[0] {
-        ResponseData::ProcStart { id } => *id,
+        ResponseData::ProcSpawned { id } => *id,
         x => panic!("Unexpected response: {:?}", x),
     };
 
@@ -1826,7 +1839,8 @@ async fn proc_list_should_send_proc_entry_list(#[future] session: Session) {
             entries: vec![RunningProcess {
                 cmd: SCRIPT_RUNNER.to_string(),
                 args: vec![SLEEP_SH.to_str().unwrap().to_string(), String::from("10")],
-                detached: false,
+                persist: false,
+                pty: None,
                 id,
             }],
         },
@@ -1880,3 +1894,50 @@ async fn system_info_should_send_system_info_based_on_binary(#[future] session:
         res.payload[0]
     );
 }
+
+#[rstest]
+#[tokio::test]
+async fn watch_should_fail_as_unsupported(#[future] session: Session) {
+    let mut session = session.await;
+
+    let req = Request::new(
+        "test-tenant",
+        vec![RequestData::Watch {
+            path: PathBuf::from("/some/path"),
+            recursive: true,
+            only: Default::default(),
+            except: Default::default(),
+        }],
+    );
+    let res = session.send(req).await.unwrap();
+
+    assert_eq!(res.payload.len(), 1, "Wrong payload size");
+    match &res.payload[0] {
+        ResponseData::Error(x) => {
+            assert_eq!(x.to_string(), "Other: Unsupported");
+        }
+        x => panic!("Unexpected response: {:?}", x),
+    }
+}
+
+#[rstest]
+#[tokio::test]
+async fn unwatch_should_fail_as_unsupported(#[future] session: Session) {
+    let mut session = session.await;
+
+    let req = Request::new(
+        "test-tenant",
+        vec![RequestData::Unwatch {
+            path: PathBuf::from("/some/path"),
+        }],
+    );
+    let res = session.send(req).await.unwrap();
+
+    assert_eq!(res.payload.len(), 1, "Wrong payload size");
+    match &res.payload[0] {
+        ResponseData::Error(x) => {
+            assert_eq!(x.to_string(), "Other: Unsupported");
+        }
+        x => panic!("Unexpected response: {:?}", x),
+    }
+}
